function [saved_states, accept_rates, saved_logprob, R, counter_dez] = ...
    DEMCz(logtarget, states, Z, n, n_thin, blockindex, ...
    current_logprob, temperature_schedule, gamma_schedule, jitter)


% DEMCz Perform Differential Evolution-Markov chain Monte Carlo sampling
% using sampling from the past.
% (see References)
% NOTE: gamma_schedule default not properly implemented for blocks
%
% Usage
%
% states = DEMCz(logtarget,  initial_states, n, n_thin, blockindex,...
%       initial_logprob, temperature_schedule, gamma_schedule, jitter)
%
% [states, accept, logprob] = DEMC_sample(logtarget,  initial_states, n, n_thin, blockindex,...
%       initial_logprob, temperature_schedule, gamma_schedule, jitter)
%
% INPUTS
%
% logtarget - a function accepting a 1-by-d state vector and returning the log
% probability density for that vector of d parameters. The density may be unnormalized.
% (DEMC may fail if the target density does not have support over R^d.)
%
%
% states - an m-by-d matrix of (start position, i.e. initial) states where
% the rows index Markov chains in the population (m parallel chains) 
% and the columns index the random variables defining the state (d parameters).
%
% Z - an mZ-by-d matrix of initial samples, eg obtained by sampling from an
% approximation of the posterior density. The recommendation is to use mZ =10*d 
%
% n - number of iterates of each of the m parallel chains to sample.
%
% n*m is the number of iterations carried out
%
% OPTIONAL INPUTS (may be [], or may be left off the argument list entirely)
%
% n_thin - thinning rate (1,2,3,  ..), every n_thin-th iteration is stored
%
% blockindex - a cell array of length b, each of the b cells contains the indexes
% of state vector elements belonging to the b'th block. For DE-MC without blocks,
% set blockindex = {1:d}; this is also the default.
%
% initial_logprob - an m-by-1 vector containing the log probability of the
% initial states; if this has been pre-calculated then this can save time,
% especially if this routine is being called in a loop as part of a hybrid
% sampling scheme.
%
% temperature_schedule - schedule for temperatures used in simulated annealing.
% After the schedule is complete, the temperature is set to 1. An empty schedule
% defaults to a constant temperature of 1.
%
% gamma_schedule - matrix with b rows (one for each block) giving the
% schedule for scale factor for vector differences of each block; the
% default value is an unchanging schedule of 2.38/sqrt(2d_k), 
% with d_k the number of parameters in the b'th block,
% which is optimal for d-dimensional Gaussian targets. 
% For example, a schedule of 9 iterations at gamma = 2.38/sqrt(2.*d)
% followed by one iteration at 0.99 for a setup without blocks is specified by
% gamma = [2.38/sqrt(2.*d).*ones(1,9) 0.99]
% 
% 
%
% jitter - 1-by-d dimensional vector of scale factors for the jitter; the
% default value is 1e-5.*ones(1,d)
%
% OUTPUT
%
% states - a m-by-d-by-n array of sampled states.
%
% saved_states - an (n*m/n_thin) by d array of saved states
%
% OPTIONAL OUTPUT
%
% accept_rates - a b vector with the accept rate of each block.
%
% logprob - an m-by-n vector containing the log probabilities of the
% final states of each chain.
%
% References
%
% ter Braak, C. J. F. (2006). A Markov Chain Monte Carlo version of the
% genetic algorithm Differential Evolution: easy Bayesian computing for
% real parameter spaces. Statistics and Computing, 16, 239-249.
% DOI 10.1007/s11222-006-8769-1
%
% ter  Braak C. J. F., and Vrugt J. A. (2008). Differential Evolution Markov
% Chain with snooker updater and fewer chains. Statistics and Computing 18, 435-446. 
% DOI 10.1007/s11222-008-9104-9
%
%

%% AUTHOR: Corey Yanofsky
%% $DATE: 08-Dec-2007 00:30:16 $
%% $Revision: 1.3 $
%% DEVELOPED: 7.1.0.246 (R14) Service Pack 3
%% FILENAME: block_DEMC_sample.m
%%
%% This work was supported by grants from Genome Quebec and the National
%% Science and Engineering Research Council of Canada.
%%
%% Changed by Simon van Mourik and Cajo ter Braak
%% October 2013



% initialize and set defaults
[m d] = size(states);
mZ0 = size(Z,1); % number of initial states
if nargin < 10 || isempty(jitter), jitter = 1e-5.*ones(1,d);end

if nargin < 8, temperature_schedule = [];end
if(nargin < 7 ||isempty(current_logprob))
    current_logprob = zeros(m, 1);
    for j = 1:m
        current_logprob(j) = logtarget(states(j,:));
    end
end
if nargin < 6 || isempty(blockindex), blockindex = {1:d};end
nblocks = length(blockindex);

if nargin < 9 || isempty(gamma_schedule)
    gamma_schedule = zeros(nblocks,1);
    for k = 1:nblocks
    gamma_schedule(k,1) = 2.38./sqrt(2*length(blockindex{k}));
    end % blocks
end

if nargin < 5 || isempty(n_thin),n_thin = 1; end


% fill out temperature schedule
temperature_schedule = temperature_schedule(:);
temperature_schedule = [temperature_schedule;ones(n - length(temperature_schedule),1)];


% create gamma index
gamidx = mod(0:(n-1),size(gamma_schedule,2)) + 1;

saved_states = zeros(m*n,d); Sequence=zeros(n,d,m);
R=zeros(round(n/500),d); nR=1;

accept_all = zeros(nblocks);

Y=1; counter_dez=0;

% sample new states n times...
for i = 1:n
    % for each of the m chains...
    for j = 1:m
        %  for each block
        for k = 1:nblocks
            mZ = size(Z,1); mZ = mZ-m;
            [newstate  accept current_logprob(j)] = DEMC_chain_sample(...
                logtarget, states(j,:),  Z, current_logprob(j), blockindex{k},...
                temperature_schedule(i), gamma_schedule(k ,gamidx(i)), jitter, mZ);
            accept_all(k) = accept_all(k) + accept;
            counter_dez=counter_dez+1;
            states(j,:)=newstate;
        end % blocks
    end % chains
    
    Sequence(i,:,:)=states';
    
    %%% Gelman's R statistic
    if mod(i,500)==0
        MCMCPar.n = d;% Nr. of parameters
        MCMCPar.seq = m; % Nr. of chains
        R(nR,:) = Gelman(Sequence(round(i/2):i,1:MCMCPar.n,1:MCMCPar.seq),MCMCPar);
        nR=nR+1;
    end
    
    if mod(i,5000)==0
        disp(['n.o. iterations ' num2str(counter_dez)]);
    end
    if  mod(i,n_thin)==0;
        Z=cat(1,Z,states);
        Y=cat(1,Y,current_logprob);
    end
    
end % iterates

saved_states=Z;
saved_logprob=Y;

saved_states(1:mZ0+1,:)=[]; %discard initial Z
saved_logprob(1)=[]; %discard initial Y

accept_rates = accept_all / (n*m);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [new_state  accept new_logprob] = DEMC_chain_sample...
    (logtarget,state ,Z, current_logprob, ...
    blockindex, temperature, gamma, jitter, mZ)




%
% This function does the actual Metropolis sampling.
%
% [new_state, accept, new_logprob] = DEMC_chain_sample(logtarget,state,Z, current_logprob, blockindex, temperature, gamma, jitter,mZ)
%
% INPUTS
%
% logtarget - a function accepting a 1-by-d state vector and returning the log
% probability density for that vector. The density may be unnormalized.
% (DEMC may fail if the target density does not have support over over R^d.)
%
% states - an m-by-d matrix of states where the rows index Markov chains in
% the population and the columns index the random variables defining the
% state.
%
% i - the index of the Markov chain being updated
%
% current_logprob - log probability of the state being updated; if this has
% been pre-calculated (for example, in a previous iteration of DE-MC), then
% this saves time
%
% blockindex - array containing the indices of the elements of the state
% vectors being updated
%
% temperature - simulated annealing temperature parameter
%
% gamma - scale factor for DE-MC; the default value is 2.38/sqrt(2d), which
% is optimal for d-dimensional Gaussian targets
%
% jitter - 1-by-d dimensional vector of scale factors for the jitter; the
% default value is 1e-5.*ones(1,d)
%
% mZ = size(Z,1) - m with the number of parallel chains
%
% OUTPUT
%
% new_state - a 1-by-d dimensional state vector sampled by DE-MC.
%
% new_logprob - the log probability of new_state. This will probably not be
% useful when the DE-MC is providing a conditional update as part of a
% hybrid Markov chain; however, if DE-MC is being used for all
% updates, then this can be fed back into the next iteration, saving
% computation time.

d = size(Z,2);

% sample two states  uniformly without replacement from Z
n1 = ceil(rand*(mZ));
n2 = ceil(rand*(mZ-1));

if(n2 >= n1), n2 = n2 + 1;end

% generate proposal via differential evolution rule
difference_vector = zeros(1,d);
difference_vector(blockindex) = gamma.*(Z(n1,blockindex) - Z(n2,blockindex)) + ...
    jitter(blockindex).*randn(1,length(blockindex));
proposal = state + difference_vector;
proposal_logprob = logtarget(proposal);


% Metropolis accept/reject step
if (temperature*log(rand) < proposal_logprob - current_logprob)
    new_state = proposal;
    new_logprob = proposal_logprob;
    accept = 1;
else
    new_state = state;
    new_logprob = current_logprob;
    accept = 0;
end

% Created with NEWFCN.m by Frank Gonzlez-Morphy
% Contact...: frank.gonzalez-morphy@mathworks.de
%% Changed by Simon van Mourik, May 2013
% ===== EOF ====== [DEMC_sample.m] ======
