
% function xdot = Goldbeter(time,x)
%  synopsis:
%     xdot = Goldbeter1991_MinMitOscil (time, x)
%     x0 = Goldbeter1991_MinMitOscil
%
%  Goldbeter1991_MinMitOscil can be used with odeN functions as follows:
%
%  x0 = Goldbeter1991_MinMitOscil;
%  [t,x] = ode23s(@Goldbeter1991_MinMitOscil, [0 100], Goldbeter1991_MinMitOscil);
%  plot (t,x);
%
%  where 100 is the end time
%
%  When Goldbeter1991_MinMitOscil is used without any arguments it returns a vector of
%  the initial concentrations of the 3 floating species.
%  Otherwise Goldbeter1991_MinMitOscil should be called with two arguments:
%  time and x.  time is the current time. x is the vector of the
%  concentrations of the 3 floating species.
%  When these parameters are supplied Goldbeter1991_MinMitOscil returns a vector of
%  the rates of change of the concentrations of the 3 floating species.
%
%  the following table shows the mapping between the vector
%  index of a floating species and the species name.
%
%  NOTE for compartmental models
%  matlab translator generates code that when simulated in matlab,
%  produces results which have the units of species amounts. Users
%  should divide the results for each species with the volume of the
%  compartment it resides in, in order to obtain concentrations.
%
%  Indx      Name
%  x(1)        C
%  x(2)        M
%  x(3)        X

% xdot = zeros(3, 1);

% List of Compartments
vol__cell = 2;		%cell

% Global Parameters
g_p1 = 0;		% V1
g_p2 = 0;		% V3
g_p3 = 3;		% VM1
g_p4 = 1;		% VM3
g_p5 = 0.5;		% Kc

% Local Parameters
% par_r1_p1 = 0.025;		% [reaction1, vi]
% par_r2_p1 = 0.01;		% [reaction2, kd]
% par_r3_p1 = 0.25;		% [reaction3, vd]
% par_r3_p2 = 0.02;		% [reaction3, Kd]
% par_r4_p1 = 0.005;		% [reaction4, K1]
% par_r5_p1 = 1.5;		% [reaction5, V2]
% par_r5_p2 = 0.005;		% [reaction5, K2]
% par_r6_p1 = 0.005;		% [reaction6, K3]
% par_r7_p1 = 0.005;		% [reaction7, K4]
% par_r7_p2 = 0.5;		% [reaction7, V4]

par_r1_p1 = a(1);%0.025;		% [reaction1, vi]
par_r2_p1 = a(2);%0.01;		% [reaction2, kd]
par_r3_p1 = a(3);%0.25;		% [reaction3, vd]
par_r3_p2 = a(4);%0.02;		% [reaction3, Kd]
par_r4_p1 = a(5); %0.005;		% [reaction4, K1]
par_r5_p1 = a(6);		% [reaction5, V2]
par_r5_p2 = a(7);		% [reaction5, K2]
par_r6_p1 = a(8);		% [reaction6, K3]
par_r7_p1 = a(9);		% [reaction7, K4]
par_r7_p2 = a(10);		% [reaction7, V4]


if (nargin == 0)
    
    % set initial conditions
    f(1) = 0.01*vol__cell;		% C = Cyclin [Concentration]
    f(2) = 0.01*vol__cell;		% M = CDC-2 Kinase [Concentration]
    f(3) = 0.01*vol__cell;		% X = Cyclin Protease [Concentration]
    
else
    
    % listOfRules
    g_p1 = (x(1))*g_p3*pow((x(1))+g_p5,-1);
    g_p2 = (x(2))*g_p4;
    
    % calculate rates of change
    R0 = vol__cell*par_r1_p1;
    R1 = (x(1))*vol__cell*par_r2_p1;
    R2 = (x(1))*vol__cell*par_r3_p1*(x(3))*pow((x(1))+par_r3_p2,-1);
    R3 = vol__cell*(1+-1*(x(2)))*g_p1*pow(par_r4_p1+-1*(x(2))+1,-1);
    R4 = vol__cell*(x(2))*par_r5_p1*pow(par_r5_p2+(x(2)),-1);
    R5 = vol__cell*g_p2*(1+-1*(x(3)))*pow(par_r6_p1+-1*(x(3))+1,-1);
    R6 = vol__cell*par_r7_p2*(x(3))*pow(par_r7_p1+(x(3)),-1);
    
    f = [
        + R0 - R1 - R2
        + R3 - R4
        + R5 - R6
        ];
end;



