% function f = Ma2002_cAMP_oscillations(time,x)
%  synopsis:
%     f = Ma2002_cAMP_oscillations (time, x)
%     x0 = Ma2002_cAMP_oscillations
%
%  Ma2002_cAMP_oscillations can be used with odeN functions as follows:
%
%  x0 = Ma2002_cAMP_oscillations;
%  [t,x] = ode23s(@Ma2002_cAMP_oscillations, [0 100], Ma2002_cAMP_oscillations);
%  plot (t,x);
%
%  where 100 is the end time
%
%  When Ma2002_cAMP_oscillations is used without any arguments it returns a vector of
%  the initial concentrations of the 7 floating species.
%  Otherwise Ma2002_cAMP_oscillations should be called with two arguments:
%  time and x.  time is the current time. x is the vector of the
%  concentrations of the 7 floating species.
%  When these parameters are supplied Ma2002_cAMP_oscillations returns a vector of 
%  the rates of change of the concentrations of the 7 floating species.
%
%  the following table shows the mapping between the vector
%  index of a floating species and the species name.
%  
%  NOTE for compartmental models
%  matlab translator generates code that when simulated in matlab, 
%  produces results which have the units of species amounts. Users 
%  should divide the results for each species with the volume of the
%  compartment it resides in, in order to obtain concentrations.
%  
%  Indx      Name
%  x(1)        ACA
%  x(2)        CAR1
%  x(3)        PKA
%  x(4)        incAMP
%  x(5)        ERK2
%  x(6)        REGA
%  x(7)        excAMP

f = zeros(7, 1);

% List of Compartments 
vol__compartment = 1;		%compartment

% Global Parameters 
% g_p1 = 2;		% k1
% g_p2 = 0.9;		% k2
% g_p3 = 2.5;		% k3
% g_p4 = 1.5;		% k4
% g_p5 = 0.6;		% k5
% g_p6 = 0.8;		% k6
% g_p7 = 1;		% k7
% g_p8 = 1.3;		% k8
% g_p9 = 0.3;		% k9
% g_p10 = 0.8;		% k10
% g_p11 = 0.7;		% k11
% g_p12 = 4.9;		% k12
% g_p13 = 23;		% k13
% g_p14 = 4.5;		% k14

g_p1 = a(1);		% k1
g_p2 = a(2);		% k2
g_p3 = a(3);		% k3
g_p4 = a(4);		% k4
g_p5 = a(5);		% k5
g_p6 = a(6);		% k6
g_p7 = a(7);		% k7
g_p8 = a(8);		% k8
g_p9 = a(9);		% k9
g_p10 = a(10);		% k10
g_p11 = a(11);		% k11
g_p12 = a(12);		% k12
g_p13 = a(13);		% k13
g_p14 = a(14);		% k14

% Local Parameters

if (nargin == 0)

    % set initial conditions
   f(1) = 3.39*vol__compartment;		% ACA = ACA [Concentration]
   f(2) = 2.45*vol__compartment;		% CAR1 = CAR1 [Concentration]
   f(3) = 1.6*vol__compartment;		% PKA = PKA [Concentration]
   f(4) = 1.2*vol__compartment;		% incAMP = incAMP [Concentration]
   f(5) = 1.13*vol__compartment;		% ERK2 = ERK2 [Concentration]
   f(6) = 0.9*vol__compartment;		% REGA = REGA [Concentration]
   f(7) = 0.48*vol__compartment;		% excAMP = excAMP [Concentration]

else

    % calculate rates of change
   R0 = g_p1*(x(2));
   R1 = g_p2*(x(1))*(x(3));
   R2 = g_p3*(x(4));
   R3 = g_p4*(x(3));
   R4 = g_p5*(x(2));
   R5 = g_p6*(x(3))*(x(5));
   R6 = g_p7;
   R7 = g_p8*(x(5))*(x(6));
   R8 = g_p9*(x(1));
   R9 = g_p10*(x(6))*(x(4));
   R10 = g_p11*(x(1));
   R11 = g_p12*(x(7));
   R12 = g_p13*(x(7));
   R13 = g_p14*(x(2));

   f = [
      + R0 - R1
      + R12 - R13
      + R2 - R3
      + R8 - R9
      + R4 - R5
      + R6 - R7
      + R10 - R11
   ];
end;
