% function f = Leloup1999_CircClock_periodic(time,x)
%  synopsis:
%     f = Leloup1999_CircClock_periodic (time, x)
%     x0 = Leloup1999_CircClock_periodic
%
%  Leloup1999_CircClock_periodic can be used with odeN functions as follows:
%
%  x0 = Leloup1999_CircClock_periodic;
%  [t,x] = ode23s(@Leloup1999_CircClock_periodic, [0 100], Leloup1999_CircClock_periodic);
%  plot (t,x);
%
%  where 100 is the end time
%
%  When Leloup1999_CircClock_periodic is used without any arguments it returns a vector of
%  the initial concentrations of the 10 floating species.
%  Otherwise Leloup1999_CircClock_periodic should be called with two arguments:
%  time and x.  time is the current time. x is the vector of the
%  concentrations of the 10 floating species.
%  When these parameters are supplied Leloup1999_CircClock_periodic returns a vector of 
%  the rates of change of the concentrations of the 10 floating species.
%
%  the following table shows the mapping between the vector
%  index of a floating species and the species name.
%  
%  NOTE for compartmental models
%  matlab translator generates code that when simulated in matlab, 
%  produces results which have the units of species amounts. Users 
%  should divide the results for each species with the volume of the
%  compartment it resides in, in order to obtain concentrations.
%  
%  Indx      Name
%  x(1)        P0
%  x(2)        T0
%  x(3)        P1
%  x(4)        T1
%  x(5)        P2
%  x(6)        T2
%  x(7)        CC
%  x(8)        Cn
%  x(9)        Mp
%  x(10)        Mt

f = zeros(10, 1);

% List of Compartments 
vol__Cell = 1;		%cytoplasm
vol__compartment_0000002 = 1;		%nucleus

% Global Parameters 
g_p1 = 0;		% Pt
g_p2 = 0;		% Tt
g_p3 = 0.7;		% V_mT
g_p4 = 2;		% V_dT

% Local Parameters
% par_r1_p1 = 2;		% [P0_to_P1, K1_P]
% par_r1_p2 = 8;		% [P0_to_P1, V_1P]
% par_r2_p1 = 2;		% [T0_to_T1, K_1T]
% par_r2_p2 = 8;		% [T0_to_T1, V_1T]
% par_r3_p1 = 2;		% [P1_to_P0, K_2P]
% par_r3_p2 = 1;		% [P1_to_P0, V_2P]
% par_r4_p1 = 2;		% [T1_to_T0, K_2T]
% par_r4_p2 = 1;		% [T1_to_T0, V_2T]
% par_r5_p1 = 2;		% [P1_to_P2, K_3P]
% par_r5_p2 = 8;		% [P1_to_P2, V_3P]
% par_r6_p1 = 2;		% [T1_to_T2, K_3T]
% par_r6_p2 = 8;		% [T1_to_T2, V_3T]
% par_r7_p1 = 2;		% [P2_to_P1, K_4P]
% par_r7_p2 = 1;		% [P2_to_P1, V_4P]
% par_r8_p1 = 2;		% [T2_to_T1, K_4T]
% par_r8_p2 = 1;		% [T2_to_T1, V_4T]
% par_r9_p1 = 0.01;		% [P0_degradation, k_d]
% par_r10_p1 = 0.01;		% [T0_degradation, k_d]
% par_r11_p1 = 0.01;		% [P1_degradation, k_d]
% par_r12_p1 = 0.01;		% [T1_degradation, k_d]
% par_r13_p1 = 0.01;		% [P2_degradation, k_d]
% par_r13_p2 = 2;		% [P2_degradation, V_dP]
% par_r13_p3 = 0.2;		% [P2_degradation, K_dP]
% par_r14_p1 = 0.01;		% [T2_degradation, k_d]
% par_r14_p2 = 0.2;		% [T2_degradation, K_dT]
% par_r15_p1 = 1.2;		% [PT_complex_formation, k3]
% par_r15_p2 = 0.6;		% [PT_complex_formation, k4]
% par_r16_p1 = 0.6;		% [PT_complex_nucleation, k1]
% par_r16_p2 = 0.2;		% [PT_complex_nucleation, k2]
% par_r17_p1 = 0.01;		% [PT_complex_degradation, k_dC]
% par_r18_p1 = 0.01;		% [PTnucl_complex_degradation, k_dN]
% par_r19_p1 = 1;		% [Mp_production, v_sP]
% par_r19_p2 = 1;		% [Mp_production, K_IP]
% par_r19_p3 = 4;		% [Mp_production, n]
% par_r20_p1 = 1;		% [Mt_production, V_sT]
% par_r20_p2 = 1;		% [Mt_production, K_IT]
% par_r20_p3 = 4;		% [Mt_production, n]
% par_r21_p1 = 0.9;		% [P0_production, k_sP]
% par_r22_p1 = 0.9;		% [T0_production, k_sT]
% par_r23_p1 = 0.01;		% [Mp_degradation, k_d]
% par_r23_p2 = 0.7;		% [Mp_degradation, V_mP]
% par_r23_p3 = 0.2;		% [Mp_degradation, K_mP]
% par_r24_p1 = 0.01;		% [Mt_degradation, k_d]
% par_r24_p2 = 0.2;		% [Mt_degradation, K_mT]

a0=[2 8 2 8 2 1 2 1 2 8 2 8 2 1 2 1 0.01 0.01 0.01 0.01 0.01 2 0.2 0.01...
    0.2 1.2 0.6 0.6 0.2 0.01 0.01 1 1 4 1 1 4 0.9 0.9 0.01 0.7 0.2 0.01...
    0.2]';

par_r1_p1 = a(1);		% [P0_to_P1, K1_P]
par_r1_p2  = a(2);		% [P0_to_P1, V_1P]
par_r2_p1  = a(3);		% [T0_to_T1, K_1T]
par_r2_p2  = a(4);		% [T0_to_T1, V_1T]
par_r3_p1  = a(5);		% [P1_to_P0, K_2P]
par_r3_p2  = a(6);		% [P1_to_P0, V_2P]
par_r4_p1  = a(7);		% [T1_to_T0, K_2T]
par_r4_p2  = a(8);		% [T1_to_T0, V_2T]
par_r5_p1  = a(9);		% [P1_to_P2, K_3P]
par_r5_p2  = a(10);		% [P1_to_P2, V_3P]
par_r6_p1  = a(11);		% [T1_to_T2, K_3T]
par_r6_p2  = a(12);		% [T1_to_T2, V_3T]
par_r7_p1  = a(13);		% [P2_to_P1, K_4P]
par_r7_p2  = a(14);		% [P2_to_P1, V_4P]
par_r8_p1  = a(15);		% [T2_to_T1, K_4T]
par_r8_p2  = a(16);		% [T2_to_T1, V_4T]
par_r9_p1  = a(17);		% [P0_degradation, k_d]
par_r10_p1  = a(18);		% [T0_degradation, k_d]
par_r11_p1  = a(19);		% [P1_degradation, k_d]
par_r12_p1  = a(20);		% [T1_degradation, k_d]
par_r13_p1  = a(21);		% [P2_degradation, k_d]
par_r13_p2  = a(22);		% [P2_degradation, V_dP]
par_r13_p3  = a(23);		% [P2_degradation, K_dP]
par_r14_p1  = a(24);		% [T2_degradation, k_d]
par_r14_p2  = a(25);		% [T2_degradation, K_dT]
par_r15_p1  = a(26);		% [PT_complex_formation, k3]
par_r15_p2  = a(27);		% [PT_complex_formation, k4]
par_r16_p1  = a(28);		% [PT_complex_nucleation, k1]
par_r16_p2  = a(29);		% [PT_complex_nucleation, k2]
par_r17_p1  = a(30);		% [PT_complex_degradation, k_dC]
par_r18_p1  = a(31);		% [PTnucl_complex_degradation, k_dN]
par_r19_p1  = a(32);		% [Mp_production, v_sP]
par_r19_p2  = a(33);		% [Mp_production, K_IP]
par_r19_p3  = a(34);		% [Mp_production, n]
par_r20_p1  = a(35);		% [Mt_production, V_sT]
par_r20_p2  = a(36);		% [Mt_production, K_IT]
par_r20_p3  = a(37);		% [Mt_production, n]
par_r21_p1  = a(38);		% [P0_production, k_sP]
par_r22_p1  = a(39);		% [T0_production, k_sT]
par_r23_p1  = a(40);		% [Mp_degradation, k_d]
par_r23_p2  = a(41);		% [Mp_degradation, V_mP]
par_r23_p3  = a(42);		% [Mp_degradation, K_mP]
par_r24_p1  = a(43);		% [Mt_degradation, k_d]
par_r24_p2  = a(44);		% [Mt_degradation, K_mT]

if (nargin == 0)

    % set initial conditions
   f(1) = 0*vol__Cell;		% P0 = PER Protein (unphosphorylated) [Concentration]
   f(2) = 0*vol__Cell;		% T0 = TIM Protein (unphosphorylated) [Concentration]
   f(3) = 0*vol__Cell;		% P1 = PER Protein (mono-phosphorylated) [Concentration]
   f(4) = 0*vol__Cell;		% T1 = TIM Protein (mono-phosphorylated) [Concentration]
   f(5) = 0*vol__Cell;		% P2 = PER Protein (bi-phosphorylated) [Concentration]
   f(6) = 0*vol__Cell;		% T2 = TIM Protein (bi-phosphorylated) [Concentration]
   f(7) = 0*vol__Cell;		% CC = Cytosolic PER-TIM Complex [Concentration]
   f(8) = 0*vol__compartment_0000002;		% Cn = Nuclear PER-TIM Complex [Concentration]
   f(9) = 0*vol__Cell;		% Mp = PER mRNA [Concentration]
   f(10) = 0*vol__Cell;		% Mt = TIM mRNA [Concentration]

else

    % listOfRules
   g_p1 = (x(7))+(x(8))+(x(1))+(x(3))+(x(5));
   g_p2 = (x(7))+(x(8))+(x(2))+(x(4))+(x(6));

    % calculate rates of change
   R0 = vol__Cell*par_r1_p2*(x(1))/(par_r1_p1+(x(1)));
   R1 = vol__Cell*par_r2_p2*(x(2))/(par_r2_p1+(x(2)));
   R2 = vol__Cell*par_r3_p2*(x(3))/(par_r3_p1+(x(3)));
   R3 = vol__Cell*par_r4_p2*(x(4))/(par_r4_p1+(x(4)));
   R4 = vol__Cell*par_r5_p2*(x(3))/(par_r5_p1+(x(3)));
   R5 = vol__Cell*par_r6_p2*(x(4))/(par_r6_p1+(x(4)));
   R6 = vol__Cell*par_r7_p2*(x(5))/(par_r7_p1+(x(5)));
   R7 = vol__Cell*par_r8_p2*(x(6))/(par_r8_p1+(x(6)));
   R8 = vol__Cell*par_r9_p1*(x(1));
   R9 = vol__Cell*par_r10_p1*(x(2));
   R10 = vol__Cell*par_r11_p1*(x(3));
   R11 = vol__Cell*par_r12_p1*(x(4));
   R12 = vol__Cell*par_r13_p1*(x(5))+vol__Cell*par_r13_p2*(x(5))/(par_r13_p3+(x(5)));
   R13 = vol__Cell*par_r14_p1*(x(6))+vol__Cell*g_p4*(x(6))/(par_r14_p2+(x(6)));
   R14 = vol__Cell*par_r15_p1*(x(5))*(x(6))-vol__Cell*par_r15_p2*(x(7));
   R15 = vol__Cell*par_r16_p1*(x(7))-vol__compartment_0000002*par_r16_p2*(x(8));
   R16 = vol__Cell*par_r17_p1*(x(7));
   R17 = vol__compartment_0000002*par_r18_p1*(x(8));
   R18 = vol__Cell*par_r19_p1*pow(par_r19_p2,par_r19_p3)/(pow(par_r19_p2,par_r19_p3)+pow((x(8)),par_r19_p3));
   R19 = vol__Cell*par_r20_p1*pow(par_r20_p2,par_r20_p3)/(pow(par_r20_p2,par_r20_p3)+pow((x(8)),par_r20_p3));
   R20 = vol__Cell*par_r21_p1*(x(9));
   R21 = vol__Cell*par_r22_p1*(x(10));
   R22 = vol__Cell*par_r23_p1*(x(9))+vol__Cell*par_r23_p2*(x(9))/(par_r23_p3+(x(9)));
   R23 = vol__Cell*par_r24_p1*(x(10))+vol__Cell*g_p3*(x(10))/(par_r24_p2+(x(10)));

   f = [
      - R0 + R2 - R8 + R20
      - R1 + R3 - R9 + R21
      + R0 - R2 - R4 + R6 - R10
      + R1 - R3 - R5 + R7 - R11
      + R4 - R6 - R12 - R14
      + R5 - R7 - R13 - R14
      + R14 - R15 - R16
      + R15 - R17
      + R18 - R22
      + R19 - R23
   ];
end;

