function xdot = MAPK_in_Solution(time,x)
%  synopsis:
%     xdot = MAPK_in_Solution (time, x)
%     x0 = MAPK_in_Solution
%
%  MAPK_in_Solution can be used with odeN functions as follows:
%
%  x0 = MAPK_in_Solution;
%  [t,x] = ode23s(@MAPK_in_Solution, [0 100], MAPK_in_Solution);
%  plot (t,x);
%
%  where 100 is the end time
%
%  When MAPK_in_Solution is used without any arguments it returns a vector of
%  the initial concentrations of the 22 floating species.
%  Otherwise MAPK_in_Solution should be called with two arguments:
%  time and x.  time is the current time. x is the vector of the
%  concentrations of the 22 floating species.
%  When these parameters are supplied MAPK_in_Solution returns a vector of 
%  the rates of change of the concentrations of the 22 floating species.
%
%  the following table shows the mapping between the vector
%  index of a floating species and the species name.
%  
%  NOTE for compartmental models
%  matlab translator generates code that when simulated in matlab, 
%  produces results which have the units of species amounts. Users 
%  should divide the results for each species with the volume of the
%  compartment it resides in, in order to obtain concentrations.
%  
%  Indx      Name
%  x(1)        MAPK
%  x(2)        MAPKMEKpp
%  x(3)        MAPKp
%  x(4)        MAPKPH
%  x(5)        MAPKpMAPKPH
%  x(6)        MAPKpMEKpp
%  x(7)        MAPKpp
%  x(8)        MAPKppMAPKPH
%  x(9)        MEK
%  x(10)        MEKp
%  x(11)        MEKPH
%  x(12)        MEKpMEKPH
%  x(13)        MEKpp
%  x(14)        MEKppMEKPH
%  x(15)        MEKpRAFp
%  x(16)        MEKRAFp
%  x(17)        RAF
%  x(18)        RAFK
%  x(19)        RAFp
%  x(20)        RAFPH
%  x(21)        RAFpRAFPH
%  x(22)        RAFRAFK

xdot = zeros(22, 1);

% List of Compartments 
vol__Cytoplasm = 1;		%Cytoplasm

% Local Parameters
par_r1_p1 = 1;		% [Reaction1, a1]
par_r2_p1 = 0.4;		% [Reaction2, d1]
par_r3_p1 = 0.1;		% [Reaction3, k1]
par_r4_p1 = 0.5;		% [Reaction4, a2]
par_r5_p1 = 0.5;		% [Reaction5, d2]
par_r6_p1 = 0.1;		% [Reaction6, k2]
par_r7_p1 = 3.3;		% [Reaction7, a3]
par_r8_p1 = 0.42;		% [Reaction8, d3]
par_r9_p1 = 0.1;		% [Reaction9, k3]
par_r10_p1 = 10;		% [Reaction10, a4]
par_r11_p1 = 0.8;		% [Reaction11, d4]
par_r12_p1 = 0.1;		% [Reaction12, k4]
par_r13_p1 = 3.3;		% [Reaction13, a5]
par_r14_p1 = 0.4;		% [Reaction14, d5]
par_r15_p1 = 0.1;		% [Reaction15, k5]
par_r16_p1 = 10;		% [Reaction16, a6]
par_r17_p1 = 0.8;		% [Reaction17, d6]
par_r18_p1 = 0.1;		% [Reaction18, k6]
par_r19_p1 = 20;		% [Reaction19, a7]
par_r20_p1 = 0.6;		% [Reaction20, d7]
par_r21_p1 = 0.1;		% [Reaction21, k7]
par_r22_p1 = 5;		% [Reaction22, a8]
par_r23_p1 = 0.4;		% [Reaction23, d8]
par_r24_p1 = 0.1;		% [Reaction24, k8]
par_r25_p1 = 20;		% [Reaction25, a9]
par_r26_p1 = 0.6;		% [Reaction26, d9]
par_r27_p1 = 0.1;		% [Reaction27, k9]
par_r28_p1 = 5;		% [Reaction28, a10]
par_r29_p1 = 0.4;		% [Reaction29, d10]
par_r30_p1 = 0.1;		% [Reaction30, k10]

if (nargin == 0)

    % set initial conditions
   xdot(1) = 0.4;		% MAPK = MAPK [Amount]
   xdot(2) = 0;		% MAPKMEKpp = MAPK_MEK-PP [Amount]
   xdot(3) = 0;		% MAPKp = MAPK-P [Amount]
   xdot(4) = 0.3;		% MAPKPH = MAPK phosphatase [Amount]
   xdot(5) = 0;		% MAPKpMAPKPH = MAPK-P_MAPKPase [Amount]
   xdot(6) = 0;		% MAPKpMEKpp = MAPK-P_MEK-PP [Amount]
   xdot(7) = 0;		% MAPKpp = MAPK-PP [Amount]
   xdot(8) = 0;		% MAPKppMAPKPH = MAPK-PP_MAPKPase [Amount]
   xdot(9) = 0.2;		% MEK = MEK [Amount]
   xdot(10) = 0;		% MEKp = MEK-P [Amount]
   xdot(11) = 0.2;		% MEKPH = MEK phosphatase [Amount]
   xdot(12) = 0;		% MEKpMEKPH = MEK-P_MEKPase [Amount]
   xdot(13) = 0;		% MEKpp = MEK-PP [Amount]
   xdot(14) = 0;		% MEKppMEKPH = MEK-PP_MEKPase [Amount]
   xdot(15) = 0;		% MEKpRAFp = MEK-P_RAF-P [Amount]
   xdot(16) = 0;		% MEKRAFp = MEK_RAF-P [Amount]
   xdot(17) = 0.3;		% RAF = RAF [Amount]
   xdot(18) = 0.2;		% RAFK = RAFK [Amount]
   xdot(19) = 0;		% RAFp = RAF-P [Amount]
   xdot(20) = 0.3;		% RAFPH = RAF phosphatase [Amount]
   xdot(21) = 0;		% RAFpRAFPH = RAF-P_RAFPase [Amount]
   xdot(22) = 0;		% RAFRAFK = RAF_RAFK [Amount]

else

    % calculate rates of change
   R0 = par_r1_p1*(x(17))*(x(18));
   R1 = par_r2_p1*(x(22));
   R2 = par_r3_p1*(x(22));
   R3 = par_r4_p1*(x(19))*(x(20));
   R4 = par_r5_p1*(x(21));
   R5 = par_r6_p1*(x(21));
   R6 = par_r7_p1*(x(9))*(x(19));
   R7 = par_r8_p1*(x(16));
   R8 = par_r9_p1*(x(16));
   R9 = par_r10_p1*(x(10))*(x(11));
   R10 = par_r11_p1*(x(12));
   R11 = par_r12_p1*(x(12));
   R12 = par_r13_p1*(x(10))*(x(19));
   R13 = par_r14_p1*(x(15));
   R14 = par_r15_p1*(x(15));
   R15 = par_r16_p1*(x(11))*(x(13));
   R16 = par_r17_p1*(x(14));
   R17 = par_r18_p1*(x(14));
   R18 = par_r19_p1*(x(1))*(x(13));
   R19 = par_r20_p1*(x(2));
   R20 = par_r21_p1*(x(2));
   R21 = par_r22_p1*(x(3))*(x(4));
   R22 = par_r23_p1*(x(5));
   R23 = par_r24_p1*(x(5));
   R24 = par_r25_p1*(x(3))*(x(13));
   R25 = par_r26_p1*(x(6));
   R26 = par_r27_p1*(x(6));
   R27 = par_r28_p1*(x(4))*(x(7));
   R28 = par_r29_p1*(x(8));
   R29 = par_r30_p1*(x(8));

   xdot = [
      - R18 + R19 + R23
      + R18 - R19 - R20
      + R20 - R21 + R22 - R24 + R25 + R29
      - R21 + R22 + R23 - R27 + R28 + R29
      + R21 - R22 - R23
      + R24 - R25 - R26
      + R26 - R27 + R28
      + R27 - R28 - R29
      - R6 + R7 + R11
      + R8 - R9 + R10 - R12 + R13 + R17
      - R9 + R10 + R11 - R15 + R16 + R17
      + R9 - R10 - R11
      + R14 - R15 + R16 - R18 + R19 + R20 - R24 + R25 + R26
      + R15 - R16 - R17
      + R12 - R13 - R14
      + R6 - R7 - R8
      - R0 + R1 + R5
      - R0 + R1 + R2
      + R2 - R3 + R4 - R6 + R7 + R8 - R12 + R13 + R14
      - R3 + R4 + R5
      + R3 - R4 - R5
      + R0 - R1 - R2
   ];
end;


function z = pow (x, y) 
    z = x^y; 


function z = sqr (x) 
    z = x*x; 


function z = piecewise(varargin) 
		numArgs = nargin; 
		result = 0; 
		foundResult = 0; 
		for k=1:2: numArgs-1 
			if varargin{k+1} == 1 
				result = varargin{k}; 
				foundResult = 1; 
				break; 
			end 
		end 
		if foundResult == 0 
			result = varargin{numArgs}; 
		end 
		z = result; 


function z = gt(a,b) 
   if a > b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = lt(a,b) 
   if a < b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = geq(a,b) 
   if a >= b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = leq(a,b) 
   if a <= b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = neq(a,b) 
   if a ~= b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = and(varargin) 
		result = 1;		 
		for k=1:nargin 
		   if varargin{k} ~= 1 
		      result = 0; 
		      break; 
		   end 
		end 
		z = result; 


function z = or(varargin) 
		result = 0;		 
		for k=1:nargin 
		   if varargin{k} ~= 0 
		      result = 1; 
		      break; 
		   end 
		end 
		z = result; 


function z = xor(varargin) 
		foundZero = 0; 
		foundOne = 0; 
		for k = 1:nargin 
			if varargin{k} == 0 
			   foundZero = 1; 
			else 
			   foundOne = 1; 
			end 
		end 
		if foundZero && foundOne 
			z = 1; 
		else 
		  z = 0; 
		end 
		 


function z = not(a) 
   if a == 1 
   	  z = 0; 
   else 
      z = 1; 
   end 


function z = root(a,b) 
	z = a^(1/b); 
 
