
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% SAMPLE PARAMETER PDF AND COMPUTE PREDICTION UNCERTAINTY %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% REF: Prediction uncertainty assessment of a systems biology   %%
%%% model requires a sample of the probability distribution of its %
%%% parameters                                              %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Author: Simon van Mourik %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Date, place: May 2014, Wageningen, The Netherlands %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%% Notation
%% a=theta parameter vector
%% a0=thetaPML  Penalized Maximum Likelihood parameter vector
%% states={theta} sample of parameter vectors
%% Y=y measured output
%% Y0=y(theta^PML) Penalized Maximum Likelihood prediction
%% x0= initial concentration
%% par= collection of parameters used in functions and subroutines

clear all; clc; close all

%%-----------------------------------------------------------------
%%%%%%% SETTINGS  %%%%%%%%%%%
%% General settings
nsample=100; %n.o. samples representing the PDF
samplepdf=0; %% 0=load sampled pdf, 1=run MCMC to sample pdf
par.predictions=1; %1= make predictions and compute Q_alpha (only for models 2-7)

modelset=[2:7]; 
%% set of models; model 1 = illustrative model in Fig 2
%% model 2:7 = models in Fig 3

%%% PART I: ESTIMATION OF THE POSTERIOR
%%%%% Choose MCMC=1 above
%n.o. iterations per chain, thinning rate, n.o. chains: (For explanation, see: ter Braak, C. & Vrugt, J. Differential evolution markov chain with snooker updater andfewer chains. Statistics and Computing 18, 435{446 (2008).)
nit=1e5;  nthin=10; nchain=4;
bi=nit/nthin; % number of burn-in iterations
savestates=0; %% save sampled states in .mat file
par.localapp=0; %% 1=approximate Hessian with second order Taylor expansion,
% use only when samplepdf=1
directdraw=0; %% 1=draw directly from posterior ~N(mu,2H^(-1)), use only
%% with par.localapp=1
LCA=0; %% 1=linearized covariance analysis, use only for model 1 
initialsampling=0; %% generate 10*n initial samples. 0: random, 1:
%% from normal distribution with covariance matrix H^-1 with H the Hessian
Gelman=0; %% 1=plot Gelman statistic (see: Gelman, A. & Rubin, D. B. Inference from iterative simulation using multiple sequences. Statistical science 457{472 (1992))
% use Gelman only when samplepdf=1
plotpdf=0; %% 1=plot 100(1-alpha) % credible regions
plot_Q=1; %% 1=plot Q_alpha values, use only when par.predictions=1

%%% PART II: COMPUTATION OF Q_alpha
%%%%%%% Choose par.predictions=1 above
alph=0.95; %0.95; %% confidence level x 100% of Q (gives Q_alpha)
plotdyn=0; %% 1=plot prediction dynamics, as in Fig 2

%%%%%%%%% DATA SET AND theta^PML %%%%%%%%%%%%%%%%%%

%%% load data here

%%% define or estimate theta^PML here
%% (theta^PML is used as starting point for MCMC smapling, and
%% as approximation point of the CHI2 error criterion)

for model=modelset
    
    par.model=model;
    disp(['MODEL ' num2str(model)])
        
    %%%%% SPECIFIC MODEL SETTINGS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if model==1 %% Illustrative model
        n=2; ny=1; a0=[10 1]'; Tf=5;
        z0=[1 1]; x0=z0(1); %% z0=initial conditions; z0(1)=x0 for calibration, z0(2)=x0 for validation
        par.u1=1; %u1*sin(t), used for calibration 
        par.u2=7; %u2*sin(t), used for prediction
    elseif model==2 %% Laub-Loomis
        Tf=15; n=14; ny=7; 
        a0=[2 0.9 2.5 1.5 0.6 0.8 1 1.3 0.3 0.8 0.7 4.9 23 4.5]';
        x0=[3.39 2.45 1.6 1.2 1.13 0.9 0.48]';  % initial concentration        
    elseif model==3 %% Levchenko
        Tf=100; n=30; ny=22; x0=MAPK_in_Solution; a0Levchenko;
    elseif model==4 %% Goldbeter
        Tf=25; n=10; ny=3; x0=Goldbeter1991_MinMitOscil; a0=[.025 .01 .25 .02 .005 1.5 0.005 0.005 0.005 0.5]';
    elseif model==5 %% Tyson
        Tf=50; n=8; ny=6; x0 = Tyson1991CellModel_6;  a0=[1 1e6 1e3 2e2 0.015 0.6 180 0.018]';
    elseif model==6 %% Vilar
        Tf=50; n=16; ny=9; x0=vilarOscillator; a0=[ 2 1 1 .2 1 50 50 500 10 50 1 100 0.01 50 .5 5]';
    elseif model==7 %% Leloup
        Tf=50; n=44; ny=10; x0=[ 0.0341  0.0341  0.0304  0.0304  0.0257 0.0257  0.2091 1.1551  0.1483 0.1483]';
        a0=[2 8 2 8 2 1 2 1 2 8 2 8 2 1 2 1 0.01 0.01 0.01 0.01 0.01 2 0.2 0.01 0.2 1.2 0.6 0.6 0.2 0.01 0.01 1 1 4 1 1 4 0.9 0.9 0.01 0.7 0.2 0.01 0.2]';
    end
    
    modellabel={'Illustrative' 'Laub' 'Levchenko' 'Goldbeter' 'Tyson' 'Vilar' 'Leloup'};

    par.a0=a0; par.x0=x0;
    
    %%% SEARCH DOMAIN
    delta=10^6; %% search domain: [theta^PML/delta,delta*theta^PML]
    par.lb=a0/delta;
    par.ub=a0*delta;
    
    
    %%% transform parameter and boundaries of search domain to logspace
    a0=transform(a0); par.a0=a0;
    par.lb=transform(par.lb); par.ub=transform(par.ub);
    
    %%% We here simulate the data 
    nd=10; nd2=100; par.td=linspace(0,Tf,nd); %% time points for calibration (nd)
    % and for prediction (nd2). To plot the error bar in Fig 2A, nd2=10 was used 
    par.preddynamics=0; %% used in functionf
    [time par.Y0]=makeY(a0,x0,par);
    b=0.1; sig=b*par.Y0; par.sig=sig; %% b: relative Gaussian noise level
    
    if LCA==1; computeH; end
    
    %%-----------------------------------------------------------------
    %%%%% PART I: ESTIMATION OF THE POSTERIOR %%%%%%%%%%%%%%%%%%%%%%%
    if samplepdf==1
        disp('Sampling PDF..')
        par.preddynamics=0; %% used in functionf
        
        %%%%%% SAMPLE PDF %%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%% create states (parameter vectors) that represent the parameter
        %%% distribution
        
        Makedistribution        
        
        %%%%% take a sample of the PDF
        if directdraw==0
            states_s=zeros(nsample,n);
            for j2=1:nsample
                j3=ceil(rand*size(states,1)); %% select random state number
                states_s(j2,:)=states(j3,:);
            end
        end
        
        %% save sample of the states
        if savestates==1; %% save states
            states=backtransform(states);
            save(['samples\sample_model' num2str(model)],'states')
            states=transform(states);
        end
                
    end
    
    %%-----------------------------------------------------------------
    %%% PART II: COMPUTATION OF Q_alpha (PREDICTION UNCERTAINTY)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if par.predictions==1
        disp(' ')
        disp(['Computing Q_' num2str(alph) '..'])
        par.localapp=0; par.preddynamics=1;
        
        
        if samplepdf==0
            load(['samples\sample_model' num2str(model) ])
            states=transform(states); %% transform to logspace

            %%%%% take a sample of the PDF
            states_s=zeros(nsample,n);
            for j2=1:nsample
                j3=ceil(rand*size(states,1)); %% select random state number
                states_s(j2,:)=states(j3,:);
            end
        end
        
        %%% MAKE PREDICTION
        for pn=1:2*length(a0); %% pn perturbations 
            %% change model dynamics:
            par.predictionnumber=pn;  %for prediction 'predictionnumber' the dynamics
            %% are changed in functionf.m            
            nd2=100; par.td=linspace(0,Tf,nd2); %possibly new time series
            
            %%%%% COMPUTE y(theta^PML) %%%%%%%%%%%%%%%%%%%
            
            %% we define y(theta^PML) here for illustrational purposes
            [time,par.Y0]=makeY(a0,x0,par);
            
            %%%%%% COMPUTE Q DISTRIBUTION %%%%%%%%%%%%%%%
            Q=zeros(nsample,1);
            j2=0; flagint2=1; %%=0 when integration fails
            while flagint2==1 && j2<nsample
                j2=j2+1;
                %% create sum representing the integral of (log(y/yPML))^2
                [Q(j2,1) flagint]=functionQ(states_s(j2,:)',par);
                if flagint==1; flagint2=0; end %% if time integration fails: Q_0.95=0
            end            
            Q=Q/((nd2-1)*ny); %% take the average over time and number of outputs
            
            %%%%%% COMPUTE Q_alpha %%%%%%%%%%%%%%%%%%%%%%%%%
            ind=round((1-alph)*nsample);
            [val ix]=sort(Q,'descend'); %% sort Q values
            Q(ix(1:ind),:)=[]; %% discard largest alpha% of the Q values            
            Q_alpha{model}(pn)=max(Q)*flagint2;
            
            %%% display Q_alpha
            disp(['Q_' num2str(alph) '=' num2str(Q_alpha{model}(pn))]);
        end
    end
   clear initstate 
end %model
%%-----------------------------------------------------------------
plots;


disp(' ')
disp('End of simulation')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

