% function xdot = Tyson1991CellModel_6(time,x)
%  synopsis:
%     xdot = Tyson1991CellModel_6 (time, x)
%     x0 = Tyson1991CellModel_6
%
%  Tyson1991CellModel_6 can be used with odeN functions as follows:
%
%  x0 = Tyson1991CellModel_6;
%  [t,x] = ode23s(@Tyson1991CellModel_6, [0 100], Tyson1991CellModel_6);
%  plot (t,x);
%
%  where 100 is the end time
%
%  When Tyson1991CellModel_6 is used without any arguments it returns a vector of
%  the initial concentrations of the 6 floating species.
%  Otherwise Tyson1991CellModel_6 should be called with two arguments:
%  time and x.  time is the current time. x is the vector of the
%  concentrations of the 6 floating species.
%  When these parameters are supplied Tyson1991CellModel_6 returns a vector of 
%  the rates of change of the concentrations of the 6 floating species.
%
%  the following table shows the mapping between the vector
%  index of a floating species and the species name.
%  
%  NOTE for compartmental models
%  matlab translator generates code that when simulated in matlab, 
%  produces results which have the units of species amounts. Users 
%  should divide the results for each species with the volume of the
%  compartment it resides in, in order to obtain concentrations.
%  
%  Indx      Name
%  x(1)        C2
%  x(2)        CP
%  x(3)        M
%  x(4)        pM
%  x(5)        Y
%  x(6)        YP

f = zeros(6, 1);

% List of Compartments 
vol__cell = 1;		%cell

% Boundary Conditions 
g_p1 = 0;		% EmptySet = EmptySet [Amount]
g_p2 = 0;		% YT = total_cyclin [Amount]
g_p3 = 0;		% CT = total_cdc2 [Amount]

% Local Parameters
% par_r1_p1 = 1;		% [Reaction1, k6]
% par_r2_p1 = 1e+006;		% [Reaction2, k8notP]
% par_r3_p1 = 1000;		% [Reaction3, k9]
% par_r4_p1 = 200;		% [Reaction4, k3]
% par_r5_p1 = 0;		% [Reaction5, k5notP]
% par_r6_p1 = 0.015;		% [Reaction6, k1aa]
% par_r7_p1 = 0;		% [Reaction7, k2]
% par_r8_p1 = 0.6;		% [Reaction8, k7]
% par_r9_p1 = 180;		% [Reaction9, k4]
% par_r9_p2 = 0.018;		% [Reaction9, k4prime]


par_r1_p1 = a(1);		% [Reaction1, k6]
par_r2_p1 = a(2);		% [Reaction2, k8notP]
par_r3_p1 = a(3);		% [Reaction3, k9]
par_r4_p1 = a(4);		% [Reaction4, k3]
par_r5_p1 = 0;		% [Reaction5, k5notP]
par_r6_p1 = a(5);		% [Reaction6, k1aa]
par_r7_p1 = 0;		% [Reaction7, k2]
par_r8_p1 = a(6);		% [Reaction8, k7]
par_r9_p1 = a(7);		% [Reaction9, k4]
par_r9_p2 = a(8);		% [Reaction9, k4prime]

if (nargin == 0)

    % set initial conditions
   f(1) = 0;		% C2 = cdc2k [Amount]
   f(2) = 0.75;		% CP = cdc2k-P [Amount]
   f(3) = 0;		% M = p-cyclin_cdc2 [Amount]
   f(4) = 0.25;		% pM = p-cyclin_cdc2-p [Amount]
   f(5) = 0;		% Y = cyclin [Amount]
   f(6) = 0;		% YP = p-cyclin [Amount]

else

    % listOfRules
   g_p2 = (x(5))+(x(6))+(x(3))+(x(4));
   g_p3 = (x(1))+(x(2))+(x(3))+(x(4));

    % calculate rates of change
   R0 = vol__cell*par_r1_p1*(x(3));
   R1 = vol__cell*(x(1))*par_r2_p1;
   R2 = vol__cell*(x(2))*par_r3_p1;
   R3 = vol__cell*(x(2))*par_r4_p1*(x(5));
   R4 = vol__cell*par_r5_p1*(x(3));
   R5 = vol__cell*par_r6_p1;
   R6 = vol__cell*par_r7_p1*(x(5));
   R7 = vol__cell*par_r8_p1*(x(6));
   R8 = vol__cell*(x(4))*(par_r9_p2+par_r9_p1*pow((x(3))/(g_p3),2));

   f = [
      + R0 - R1 + R2
      + R1 - R2 - R3
      - R0 - R4 + R8
      + R3 + R4 - R8
      - R3 + R5 - R6
      + R0 - R7
   ];
end;


