function xdot = Tyson1991CellModel_6(time,x)
%  synopsis:
%     xdot = Tyson1991CellModel_6 (time, x)
%     x0 = Tyson1991CellModel_6
%
%  Tyson1991CellModel_6 can be used with odeN functions as follows:
%
%  x0 = Tyson1991CellModel_6;
%  [t,x] = ode23s(@Tyson1991CellModel_6, [0 100], Tyson1991CellModel_6);
%  plot (t,x);
%
%  where 100 is the end time
%
%  When Tyson1991CellModel_6 is used without any arguments it returns a vector of
%  the initial concentrations of the 6 floating species.
%  Otherwise Tyson1991CellModel_6 should be called with two arguments:
%  time and x.  time is the current time. x is the vector of the
%  concentrations of the 6 floating species.
%  When these parameters are supplied Tyson1991CellModel_6 returns a vector of 
%  the rates of change of the concentrations of the 6 floating species.
%
%  the following table shows the mapping between the vector
%  index of a floating species and the species name.
%  
%  NOTE for compartmental models
%  matlab translator generates code that when simulated in matlab, 
%  produces results which have the units of species amounts. Users 
%  should divide the results for each species with the volume of the
%  compartment it resides in, in order to obtain concentrations.
%  
%  Indx      Name
%  x(1)        C2
%  x(2)        CP
%  x(3)        M
%  x(4)        pM
%  x(5)        Y
%  x(6)        YP

xdot = zeros(6, 1);

% List of Compartments 
vol__cell = 1;		%cell

% Boundary Conditions 
g_p1 = 0;		% EmptySet = EmptySet [Amount]
g_p2 = 0;		% YT = total_cyclin [Amount]
g_p3 = 0;		% CT = total_cdc2 [Amount]

% Local Parameters
par_r1_p1 = 1;		% [Reaction1, k6]
par_r2_p1 = 1e+006;		% [Reaction2, k8notP]
par_r3_p1 = 1000;		% [Reaction3, k9]
par_r4_p1 = 200;		% [Reaction4, k3]
par_r5_p1 = 0;		% [Reaction5, k5notP]
par_r6_p1 = 0.015;		% [Reaction6, k1aa]
par_r7_p1 = 0;		% [Reaction7, k2]
par_r8_p1 = 0.6;		% [Reaction8, k7]
par_r9_p1 = 180;		% [Reaction9, k4]
par_r9_p2 = 0.018;		% [Reaction9, k4prime]

if (nargin == 0)

    % set initial conditions
   xdot(1) = 0;		% C2 = cdc2k [Amount]
   xdot(2) = 0.75;		% CP = cdc2k-P [Amount]
   xdot(3) = 0;		% M = p-cyclin_cdc2 [Amount]
   xdot(4) = 0.25;		% pM = p-cyclin_cdc2-p [Amount]
   xdot(5) = 0;		% Y = cyclin [Amount]
   xdot(6) = 0;		% YP = p-cyclin [Amount]

else

    % listOfRules
   g_p2 = (x(5))+(x(6))+(x(3))+(x(4));
   g_p3 = (x(1))+(x(2))+(x(3))+(x(4));

    % calculate rates of change
   R0 = vol__cell*par_r1_p1*(x(3));
   R1 = vol__cell*(x(1))*par_r2_p1;
   R2 = vol__cell*(x(2))*par_r3_p1;
   R3 = vol__cell*(x(2))*par_r4_p1*(x(5));
   R4 = vol__cell*par_r5_p1*(x(3));
   R5 = vol__cell*par_r6_p1;
   R6 = vol__cell*par_r7_p1*(x(5));
   R7 = vol__cell*par_r8_p1*(x(6));
   R8 = vol__cell*(x(4))*(par_r9_p2+par_r9_p1*pow((x(3))/(g_p3),2));

   xdot = [
      + R0 - R1 + R2
      + R1 - R2 - R3
      - R0 - R4 + R8
      + R3 + R4 - R8
      - R3 + R5 - R6
      + R0 - R7
   ];
end;


function z = pow (x, y) 
    z = x^y; 


function z = sqr (x) 
    z = x*x; 


function z = piecewise(varargin) 
		numArgs = nargin; 
		result = 0; 
		foundResult = 0; 
		for k=1:2: numArgs-1 
			if varargin{k+1} == 1 
				result = varargin{k}; 
				foundResult = 1; 
				break; 
			end 
		end 
		if foundResult == 0 
			result = varargin{numArgs}; 
		end 
		z = result; 


function z = gt(a,b) 
   if a > b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = lt(a,b) 
   if a < b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = geq(a,b) 
   if a >= b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = leq(a,b) 
   if a <= b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = neq(a,b) 
   if a ~= b 
   	  z = 1; 
   else 
      z = 0; 
   end 


function z = and(varargin) 
		result = 1;		 
		for k=1:nargin 
		   if varargin{k} ~= 1 
		      result = 0; 
		      break; 
		   end 
		end 
		z = result; 


function z = or(varargin) 
		result = 0;		 
		for k=1:nargin 
		   if varargin{k} ~= 0 
		      result = 1; 
		      break; 
		   end 
		end 
		z = result; 


function z = xor(varargin) 
		foundZero = 0; 
		foundOne = 0; 
		for k = 1:nargin 
			if varargin{k} == 0 
			   foundZero = 1; 
			else 
			   foundOne = 1; 
			end 
		end 
		if foundZero && foundOne 
			z = 1; 
		else 
		  z = 0; 
		end 
		 


function z = not(a) 
   if a == 1 
   	  z = 0; 
   else 
      z = 1; 
   end 


function z = root(a,b) 
	z = a^(1/b); 
 
