% function xdot = vilarOscillator(time,x)
%  synopsis:
%     xdot = vilarOscillator (time, x)
%     x0 = vilarOscillator
%
%  vilarOscillator can be used with odeN functions as follows:
%
%  x0 = vilarOscillator;
%  [t,x] = ode23s(@vilarOscillator, [0 100], vilarOscillator);
%  plot (t,x);
%
%  where 100 is the end time
%
%  When vilarOscillator is used without any arguments it returns a vector of
%  the initial concentrations of the 9 floating species.
%  Otherwise vilarOscillator should be called with two arguments:
%  time and x.  time is the current time. x is the vector of the
%  concentrations of the 9 floating species.
%  When these parameters are supplied vilarOscillator returns a vector of
%  the rates of change of the concentrations of the 9 floating species.
%
%  the following table shows the mapping between the vector
%  index of a floating species and the species name.
%
%  NOTE for compartmental models
%  matlab translator generates code that when simulated in matlab,
%  produces results which have the units of species amounts. Users
%  should divide the results for each species with the volume of the
%  compartment it resides in, in order to obtain concentrations.
%
%  Indx      Name
%  x(1)        A
%  x(2)        C
%  x(3)        DA
%  x(4)        DAp
%  x(5)        DR
%  x(6)        DRp
%  x(7)        MA
%  x(8)        MR
%  x(9)        R

f = zeros(9, 1);

% List of Compartments
vol__deterministicOscillator = 1;		%deterministicOscillator

% Boundary Conditions
g_p1 = 0;		% EmptySet = EmptySet [Amount]

% Local Parameters
% par_r1_p1 = 2;		% [Reaction1, gammaC]
% par_r2_p1 = 1;		% [Reaction2, deltaA]
% par_r3_p1 = 1;		% [Reaction3, deltaA]
% par_r4_p1 = 0.2;		% [Reaction4, deltaR]
% par_r5_p1 = 1;		% [Reaction5, gammaA]
% par_r6_p1 = 50;		% [Reaction6, thetaA]
% par_r7_p1 = 50;		% [Reaction7, alphaA]
% par_r8_p1 = 500;		% [Reaction8, alphaAp]
% par_r9_p1 = 10;		% [Reaction9, deltaMA]
% par_r10_p1 = 50;		% [Reaction10, betaA]
% par_r11_p1 = 1;		% [Reaction11, gammaR]
% par_r12_p1 = 100;		% [Reaction12, thetaR]
% par_r13_p1 = 0.01;		% [Reaction13, alphaR]
% par_r14_p1 = 50;		% [Reaction14, alphaRp]
% par_r15_p1 = 0.5;		% [Reaction15, deltaMR]
% par_r16_p1 = 5;		% [Reaction16, betaR]



par_r1_p1 = a(1);		% [Reaction1, gammaC]
par_r2_p1 = a(2);		% [Reaction2, deltaA]
par_r3_p1 = a(3);		% [Reaction3, deltaA]
par_r4_p1 = a(4);		% [Reaction4, deltaR]
par_r5_p1 = a(5);		% [Reaction5, gammaA]
par_r6_p1 = a(6);		% [Reaction6, thetaA]
par_r7_p1 = a(7);		% [Reaction7, alphaA]
par_r8_p1 = a(8);		% [Reaction8, alphaAp]
par_r9_p1 = a(9);		% [Reaction9, deltaMA]
par_r10_p1 = a(10);		% [Reaction10, betaA]
par_r11_p1 = a(11);		% [Reaction11, gammaR]
par_r12_p1 = a(12);		% [Reaction12, thetaR]
par_r13_p1 = a(13);		% [Reaction13, alphaR]
par_r14_p1 = a(14);		% [Reaction14, alphaRp]
par_r15_p1 = a(15);		% [Reaction15, deltaMR]
par_r16_p1 = a(16);		% [Reaction16, betaR]


% 
% for npar=1:16
%     a0(npar)=eval(['par_r' num2str(npar) '_p1'],1);
% end
% a0'

if (nargin == 0)
    
    % set initial conditions
    f(1) = 0;		% A = A [Amount]      0
    f(2) = 0;		% C = C [Amount]       0
    f(3) = 1;		% DA = DA [Amount]     1
    f(4) = 0;		% DAp = DAp [Amount]   
    f(5) = 1;		% DR = DR [Amount]     1
    f(6) = 0;		% DRp = DRp [Amount]
    f(7) = 0;		% MA = MA [Amount]
    f(8) = 0;		% MR = MR [Amount]
    f(9) = 0;		% R = R [Amount]
    
else
    
    % calculate rates of change
    R0 = (x(1))*(x(9))*par_r1_p1;
    R1 = (x(1))*par_r2_p1;
    R2 = (x(2))*par_r3_p1;
    R3 = (x(9))*par_r4_p1;
    R4 = (x(1))*(x(3))*par_r5_p1;
    R5 = (x(4))*par_r6_p1;
    R6 = (x(3))*par_r7_p1;
    R7 = (x(4))*par_r8_p1;
    R8 = (x(7))*par_r9_p1;
    R9 = (x(7))*par_r10_p1;
    R10 = (x(1))*(x(5))*par_r11_p1;
    R11 = (x(6))*par_r12_p1;
    R12 = (x(5))*par_r13_p1;
    R13 = (x(6))*par_r14_p1;
    R14 = (x(8))*par_r15_p1;
    R15 = (x(8))*par_r16_p1;
    
    f = [
        - R0 - R1 - R4 + R5 + R9 - R10 + R11
        + R0 - R2
        - R4 + R5 + R6 - R6
        + R4 - R5 + R7 - R7
        - R10 + R11 + R12 - R12
        + R10 - R11 + R13 - R13
        + R6 + R7 - R8 + R9 - R9
        + R12 + R13 - R14 + R15 - R15
        - R0 + R2 - R3 + R15
        ];
end;

